﻿' This is a simple TCP and UDP server. A socket of the requested type is created that
' waits for clients. For TCP, the server waits for an incoming TCP connection after which
' it receives a "request". The request is terminated by the client shutting down the connection.
' After the request is received, the server sends data in resposne followed by shutting down its
' connection and closing the socket. The UDP server simply waits for a datagram request. The 
' request consists of a single datagram packet. The server then sends a number of responses to
' the source address of the request followed by a number of zero byte datagrams. The zero
' byte datagrams will indicate to the client that no more data will follow.
'
' usage:
'      Executable_file_namee [-l bind-address] [-m message] [-n count] [-p port]");
'                       [-t tcp|udp] [-x size]
'        -l bind-address        Local address to bind to
'        -m message             Text message to format into send buffer
'        -n count               Number of times to send a message
'        -p port                Local port to bind to
'        -t udp | tcp           Indicates which protocol to use
'        -x size                Size of send and receive buffer
'
' sample usage:
'      The following command line invokes an IPv6 TCP server bound to the wildcard address (::)
'      and port 5150 that sends 10 messages of 1024 bytes filled with the string "hola". The
'      command line following the server's is the client side that can be used to connect to
'      the server. For the client substitute the appopriate IPv6 address of the server.
'
'          Executable_file_namee -l :: -p 5150 -n 10 -m "hola" -x 1024 -t tcp
'
'          Executable_file_namee -n 3ffe::1 -p 5150 -t tcp
'
'      The following command line invokes an IPv4 UDP server bound to a specific local interface
'      on port 5150 that sends 5 messages of 512 bytes filled with the string "response". The 
'      second command line is an example of the client's command line used to connect to the server.
'      NOTE: For UDP the buffer size on client and server should match - otherwise, an exception
'            will be thrown since the smaller buffer won't be able to hold the larger received datagram.
'
'          Executable_file_namee -l 10.10.10.1 -p 5150 -n 5 -m "response" -x 512 -t udp
'
'          Executable_file_namee -n 10.10.10.1 -p 5150 -t udp -x 512
'
Imports System
Imports System.Net
Imports System.Net.Sockets

' This is a simple TCP and UDP based server.
Public Class ServerSocket
    ' Winsock ioctl code which will disable ICMP errors from being propagated to a UDP socket.
    ' This can occur if a UDP packet is sent to a valid destination but there is no socket
    ' registered to listen on the given port.
    ' http://msdn.microsoft.com/en-us/library/bb736550(VS.85).aspx
    Public Const SIO_UDP_CONNRESET = -1744830452

    ' This routine repeatedly copies a string message into a byte array until filled.        
    ' <param name="dataBuffer">Byte buffer to fill with string message</param>
    ' <param name="message">String message to copy</param>
    Shared Sub FormatBuffer(ByVal dataBuffer As Byte(), ByVal message As String)

        Dim byteMessage As Byte() = System.Text.Encoding.ASCII.GetBytes(message)
        Dim index As Integer = 0

        ' First convert the string to bytes and then copy into send buffer
        While (index < dataBuffer.GetUpperBound(0))
            Dim j As Integer
            For j = 0 To byteMessage.GetUpperBound(0) - 1
                dataBuffer(index) = byteMessage(j)
                index = index + 1

                ' Make sure we don't go past the send buffer length
                If (index >= dataBuffer.GetUpperBound(0)) Then
                    GoTo AfterFor
                End If
            Next
AfterFor:

        End While

    End Sub

    ' Prints simple usage information.        
    Shared Sub usage()
        Console.WriteLine("Executable_file_namee [-l bind-address] [-m message] [-n count] [-p port]")
        Console.WriteLine("                 [-t tcp|udp] [-x size]")
        Console.WriteLine("  -l bind-address        Local address to bind to")
        Console.WriteLine("  -m message             Text message to format into send buffer")
        Console.WriteLine("  -n count               Number of times to send a message")
        Console.WriteLine("  -p port                Local port to bind to")
        Console.WriteLine("  -t udp | tcp           Indicates which protocol to use")
        Console.WriteLine("  -x size                Size  of send and receive buffer")
        Console.WriteLine(".....Else default values will be used...")
        Console.WriteLine()
    End Sub

    ' This is the main routine that parses the command line and invokes the server with the
    ' given parameters. For TCP, it creates a listening socket and waits to accept a client
    ' connection. Once a client connects, it waits to receive a "request" message. The
    ' request is terminated by the client shutting down the connection. After the request is
    ' received, the server sends a response followed by shutting down its connection and 
    ' closing the socket. For UDP, the socket simply listens for incoming packets. The "request"
    ' message is a single datagram received. Once the request is received, a number of datagrams
    ' are sent in return followed by sending a few zero byte datagrams. This way the client
    ' can determine that the response has completed when it receives a zero byte datagram.        
    ' <param name="args">Command line arguments</param>
    Shared Sub Main()
        Dim textMessage As String = "Server: ServerResponse"
        Dim localPort As Integer = 5150
        Dim sendCount As Integer = 10
        Dim bufferSize As Integer = 4096
        Dim localAddress As IPAddress = IPAddress.Any
        Dim sockType As SocketType = SocketType.Stream
        Dim sockProtocol As ProtocolType = ProtocolType.Tcp

        usage()
        Console.WriteLine()

        ' Parse the command line
        Dim args As String() = Environment.GetCommandLineArgs()
        Dim i As Integer
        For i = 1 To args.GetUpperBound(0)
            Try
                Dim CurArg() As Char = args(i).ToCharArray(0, args(i).Length)
                If (CurArg(0) = "-") Or (CurArg(0) = "/") Then
                    Select Case Char.ToLower(CurArg(1), System.Globalization.CultureInfo.CurrentCulture)
                        Case "l" ' Local interface to bind to
                            i = i + 1
                            localAddress = IPAddress.Parse(args(i))

                        Case "m"       ' Text message to put into the send buffer
                            i = i + 1
                            textMessage = args(i)

                        Case "p"       ' Port number for the destination
                            i = i + 1
                            localPort = System.Convert.ToInt32(args(i))

                        Case "t"       ' Specified TCP or UDP
                            i = i + 1
                            If (args(i) = "tcp") Then
                                sockType = SocketType.Stream
                                sockProtocol = ProtocolType.Tcp

                            ElseIf (args(i) = "udp") Then
                                sockType = SocketType.Dgram
                                sockProtocol = ProtocolType.Udp

                            Else
                                usage()
                                Exit Sub
                            End If

                        Case "x"       ' Size of the send and receive buffers
                            i = i + 1
                            bufferSize = System.Convert.ToInt32(args(i))

                        Case Else
                            usage()
                            Exit Sub
                    End Select

                End If

            Catch e As Exception
                usage()
                Exit Sub
            End Try

        Next

        Dim serverSocket As Socket = Nothing

        Try
            Dim localEndPoint As IPEndPoint = New IPEndPoint(localAddress, localPort)
            Dim senderAddress As IPEndPoint = New IPEndPoint(localAddress, 0)
            Dim castSenderAddress As EndPoint
            Dim clientSocket As Socket
            Dim receiveBuffer(bufferSize) As Byte
            Dim sendBuffer(bufferSize) As Byte
            Dim rc As Integer

            FormatBuffer(sendBuffer, textMessage)

            ' Create the server socket
            serverSocket = New Socket( _
                localAddress.AddressFamily, _
                sockType, _
                sockProtocol _
                )
            Console.WriteLine("Server: Socket() is OK...")

            ' Bind the socket to the local interface specified
            serverSocket.Bind(localEndPoint)
            Console.WriteLine("Server: Bind() is OK...")

            Console.WriteLine("Server: {0} server socket bound to {1}", sockProtocol.ToString(), localEndPoint.ToString())

            If (sockProtocol = ProtocolType.Tcp) Then
                ' If TCP socket, set the socket to listening
                serverSocket.Listen(1)
                Console.WriteLine("Server: Listen() is OK. I'm listening for connection...")

            Else
                Dim byteTrue(4) As Byte

                ' Set the SIO_UDP_CONNRESET ioctl to true for this UDP socket. If this UDP socket
                '    ever sends a UDP packet to a remote destination that exists but there is
                '    no socket to receive the packet, an ICMP port unreachable message is returned
                '    to the sender. By default, when this is received the next operation on the
                '    UDP socket that send the packet will receive a SocketException. The native
                '    (Winsock) error that is received is WSAECONNRESET (10054). Since we don't want
                '    to wrap each UDP socket operation in a try/except, we'll disable this error
                '    for the socket with this ioctl call.
                byteTrue(byteTrue.GetUpperBound(0) - 1) = 1
                serverSocket.IOControl(SIO_UDP_CONNRESET, byteTrue, Nothing)
                Console.WriteLine("Server: IOControl() is OK...")
            End If

            ' Service clients in a loop
            While (True)

                If (sockProtocol = ProtocolType.Tcp) Then
                    ' Wait for a client connection
                    clientSocket = serverSocket.Accept()
                    Console.WriteLine("Server: Accept() is OK...")
                    Console.WriteLine("Server: Accepted connection from: {0}", clientSocket.RemoteEndPoint.ToString())

                    ' Receive the request from the client in a loop until the client shuts
                    '    the connection down via a Shutdown.
                    Console.WriteLine("Server: Ready to receive using Receive()...")
                    While (True)
                        rc = clientSocket.Receive(receiveBuffer)
                        Console.WriteLine("Server: Read {0} bytes", rc)
                        If (rc = 0) Then
                            Exit While
                        End If
                    End While

                    ' Send the indicated number of response messages
                    Console.WriteLine("Server: Ready to send using Send()...")
                    For i = 0 To sendCount - 1
                        rc = clientSocket.Send(sendBuffer)
                        Console.WriteLine("Server: Sent {0} bytes", rc)
                    Next

                    ' Shutdown the client connection
                    clientSocket.Shutdown(SocketShutdown.Send)
                    Console.WriteLine("Server: Shutdown() is OK...")

                    clientSocket.Close()
                    Console.WriteLine("Server: Close() is OK")

                Else
                    castSenderAddress = CType(senderAddress, EndPoint)

                    ' Receive the initial request from the client
                    rc = serverSocket.ReceiveFrom(receiveBuffer, castSenderAddress)
                    Console.WriteLine("Server: ReceiveFrom() is OK...")

                    senderAddress = CType(castSenderAddress, IPEndPoint)
                    Console.WriteLine("Server: Received {0} bytes from {1}", rc, senderAddress.ToString())

                    ' Send the response to the client the requested number of times
                    Console.WriteLine("Server: Ready to send using SendTo()...")
                    For i = 0 To sendCount - 1
                        Try
                            rc = serverSocket.SendTo(sendBuffer, senderAddress)
                        Catch
                            ' If the sender's address is being spoofed we may get an error when sending
                            '    the response. You can test this by using IPv6 and using the RawSocket
                            '    sample to spoof a UDP packet with an invalid link local source address.
                            GoTo ContinueSendLoop
                        End Try

                        Console.WriteLine("Server: Sent {0} bytes to {1}", rc, senderAddress.ToString())
ContinueSendLoop:
                    Next

                    ' Send several zero byte datagrams to indicate to client that no more data
                    '    will be sent from the server. Multiple packets are sent since UDP
                    '    is not guaranteed and we want to try to make an effort the client
                    '    gets at least one.
                    Console.WriteLine("Server: Ready to send using SendTo(), sleeping for Sleep(250)...")
                    For i = 0 To 2
                        serverSocket.SendTo(sendBuffer, 0, 0, SocketFlags.None, senderAddress)

                        ' Space out sending the zero byte datagrams a bit. UDP is unreliable and
                        '   the local stack can even drop them before even hitting the wire!
                        System.Threading.Thread.Sleep(250)
                    Next
                End If
            End While
        Catch err As SocketException
            Console.WriteLine("Server: Socket error occured: {0}", err.Message)
        Finally
            ' Close the socket if necessary
            If (Not IsNothing(serverSocket)) Then
                serverSocket.Close()
                Console.WriteLine("Server: Close() is OK...")
            End If
        End Try
    End Sub
End Class
